#
# Copyright 2017- ymurata
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require "fluent/parser"

module Fluent
  class TextParser
    class HerokuParser < Parser
      Plugin.register_parser("heroku-parser", self)

      # desc 'The delimiter character (or string) of TSV values'
      # config_param :delimiter, :string, default: "\t"
      # desc 'The delimiter character between field name and value'
      # config_param :label_delimiter, :string, default: ":"
      # config_set_default :time_key, 'time'
      def initialize
        super
      end

      def configure(conf)
        super
      end

      def parse(text)
        text =~ /^([^ ]*\s+[^ ]*\s+[^ ]*?)\s([^ ]*?)\s([a-zA-Z0-9_\/\.\-]*)\[([a-zA-Z0-9\.]+)\]?\s(.*)$/
        record = { access_time: $1, host: $2, ident: $3, pid: $4 }.merge(parse_message($5))
        return Time.now.to_i, record
      end

      # TODO: 取得するログを config で指定できるようにする
      def parse_message(msg)
        all = {}
        [
          '(sock)=(.*?)\s',
          '(at)=(.*?)\s',
          '(code)=(.*?)\s',
          '(desc)="(.*?)"\s',
          '(method)=(.+?)\s',
          '(path)="(.+?)"\s',
          '(host)=(.+?)\s',
          '(request_id)=(.+?)\s',
          '(fwd)="([0-9\.]+?)"\s',
          '(dyno)=(.+?)\s',
          '(connect)=([0-9]+?)ms\s',
          '(service)=([0-9]+?)ms\s',
          '(status)=([0-9]+?)\s',
          '(bytes)=([0-9]+?)\s',
          '(tls_version)=(.+)',
        ].each{|reg_str|
          all = all.merge(to_hash(msg, reg_str))
        }
        all = all.merge(parse_path(msg))
      end

      def parse_path(msg)
        p = msg.scan(/path="(.+?)"\s/).map{|v| v[0].split('?')}.flatten
        p.length == 2 ? {url: p[0], query: p[1]} : {url: p[0]}
      end

      def to_hash(msg, regexp_string)
        msg.scan(/#{regexp_string}/).map{|k, v| [k.to_sym,v.to_s]}.to_h
      end
    end
    register_template('heroku-parser', Proc.new { HerokuParser.new })
  end
end
